--
-- Loudmouth
-- Author: FatherJim of Blackhand
-- Email:  praetor@slayernet.org
--

--
-- Set up basic ACE2 library and Fubar data structures and methods.
--

local L = AceLibrary("AceLocale-2.2"):new("Loudmouth")
local tablet = AceLibrary("Tablet-2.0")
Loudmouth = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "AceEvent-2.0", "FuBarPlugin-2.0")
Loudmouth:RegisterDB("LoudmouthDB","LoudmouthDBPC")
local crayon = AceLibrary("Crayon-2.0")

--Loudmouth.date="12/29/2006"
--Loudmouth.version="0.7beta"

--
-- Set up the shared SavedVariables.lua defaults.
--

Loudmouth:RegisterDefaults("profile", {
  --
  -- Default option values
  --
  Channel = 'Say',
  Debug = false,
  Throttle = 15,
  --
  -- Current State Information
  --
  Version = 0, -- Loudmouth.version
  _QuoteSet = 1,
  _QuoteEvent = 1,
  enabled = {
    ["Crits"] = true,
	 ["Fears"] = true,
	 ["Misses"] = true,
  },
  rates = {
    ["Crits"] = 15,
    ["Fears"] = 33,
    ["Misses"] = 50,
  },
  --
  -- Quotes and options
  --
  quotes = {
    ["Default"] = {
      ["AFK"] = {
        	[1] = "Oops, my ethernet cable fell out, and the bits are making a mess on the carpet... brb",
			[2] = "Got a phone call. I won the Lottery. Back after I buy Blizzard to fix a few things...",
      },
      ["Crits"] = {
        	[1] = "I just got a critical hit!",
			[2] = "Oops, my weapon just got stuck in you, could you hand that back please?",
      },
      ["Excuses"] = {
      	[1] = "My g/f just called and wife picked up the phone. Sorry, in combat IRL!"
      },
      ["Fears"] = {
        	[1] = "Hey mob! I didn't say you could leave yet...",
			[2] = "Oh sure, fight me until you think you're gonna lose, then run away like a little girlie...",
      },
      ["Misses"] = {
        	[1] = "Stand still or come closer so I can slap you!",
			[2] = "Oops... I missed.",
      },
    },
  },
  quoteopts = {
  	["Default"] = {
		["AFK"] = {
			[1] = { ["emote"] = 0 },
			[2] = { ["emote"] = 0 },
		},
		["Crits"] = {
			[1] = { ["emote"] = 0 },
			[2] = { ["emote"] = 0 },
		},
		["Excuses"] = {
			[1] = { ["emote"] = 0 },
		},
		["Fears"] = {
			[1] = { ["emote"] = 0 },
			[2] = { ["emote"] = 0 },
		},
		["Misses"] = {
			[1] = { ["emote"] = 0 },
			[2] = { ["emote"] = 0 },
		},
	},
  },
})

--
-- Set up the internal variables, menus and structures that aren't user data
--
local scrollbar = getglobal("LMFrame_QuoteList_ScrollFrame_ScrollBar")
local statement = ''
local emote = 0

Loudmouth.dropdowns={}
Loudmouth.dropdowns.Lists={};
Loudmouth.dropdowns.Lists["_QuoteEvent"] = { index = "_QuoteEvent", label = "Event Type", selected = 2, table = { [1] = "AFK", [2] = "Crits", [3] = "Excuses", [4] = "Fears", [5] = "Misses" } }
Loudmouth.dropdowns.Lists["_QuoteSet"] = { index = "_QuoteSet", label = "Quote Set", selected = 2, table = {[1] = "Default", [2] = "Char"} }
Loudmouth.dropdowns.Lists["_SendQuote"] = { index = "_SendQuote", label = "", selected = 1, table = { [1] = "Send on Channel", [2] = "Say", [3] = "Yell", [4] = "Party", [5] = "Raid", [6] = "Guild", [7] = "Officer" } }
Loudmouth.editing_item=0
Loudmouth.hasIcon = true
Loudmouth.hideMenuTitle = true
Loudmouth.LastQuoted = 0
Loudmouth.lastQuoteID = {
	 ["AFK"] = 0,
    ["Crits"] = 0,
	 ["Excuses"] = 0,
	 ["Fears"] = 0,
	 ["Misses"] = 0,
  }

local options = {
    type = 'group',
    args = {
    	header = {
	    type = 'header',
	    name = L["Loudmouth"],
	    icon = "Interface\\AddOns\\Loudmouth\\icon.tga",
	    iconHeight = 16,
	    iconWidth = 16,
	    order = 1,
	},
        --msg = {
	    --type = 'text',
	    --name = L["Message"],
	    --desc = L["Sets the message to be displayed when you enter a new zone."],
	    --usage = L["<your message>"],
	    --get = "GetMessage",
	    --set = "SetMessage",
	--},
	crits = {
	    type = 'toggle',
	    name = L["Crit Hits"],
	    desc = L["Will you brag about critical hits?"],
	    get = function ()
  				return Loudmouth.db.profile.enabled["Crits"]
  	    end,
	    set = function ()
  				Loudmouth.db.profile.enabled["Crits"] = not Loudmouth.db.profile.enabled["Crits"]
				if (LMFrame:IsShown()) then Loudmouth:Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	critrate = {
	    type = 'range',
	    name = L["Crit Rate"],
	    desc = L["How often you brag about critical hits"],
	    min = 0,
	    max = 100,
	    step = 1,
	    isPercent = false,
	    get = function ()
	        	return Loudmouth.db.profile.rates["Crits"]
	    end,
	    set = function (v)
	         Loudmouth.db.profile.rates["Crits"] = v
				if (LMFrame:IsShown()) then Loudmouth:Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	fears = {
	    type = 'toggle',
	    name = L["Fear Effects"],
	    desc = L["Will you comment when mobs run away?"],
	    get = function ()
	        	return Loudmouth.db.profile.enabled["Fears"]
	    end,
	    set = function ()
	         Loudmouth.db.profile.enabled["Fears"] = not Loudmouth.db.profile.enabled["Fears"]
				if (LMFrame:IsShown()) then Loudmouth:Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	fearrate = {
	    type = 'range',
	    name = L["Fear Rate"],
	    desc = L["How often you taunt about Feared mobs running away"],
	    min = 0,
	    max = 100,
	    step = 1,
	    isPercent = false,
	    get = function ()
	        return Loudmouth.db.profile.rates["Fears"]
	    end,
	    set = function (v)
	        	Loudmouth.db.profile.rates["Fears"] = v
				if (LMFrame:IsShown()) then Loudmouth:Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	miss = {
	    type = 'toggle',
	    name = L["Missed Attacks"],
	    desc = L["Will you comment if you miss?"],
	    get = function ()
	        	return Loudmouth.db.profile.enabled["Misses"]
	    end,
	    set = function ()
	        	Loudmouth.db.profile.enabled["Misses"] = not Loudmouth.db.profile.enabled["Misses"]
				if (LMFrame:IsShown()) then Loudmouth:Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	missrate = {
	    type = 'range',
	    name = L["Missed Attack Rate"],
	    desc = L["How often you taunt about Missed Attacks"],
	    min = 0,
	    max = 100,
	    step = 1,
	    isPercent = false,
	    get = function ()
	        	return Loudmouth.db.profile.rates["Misses"]
	    end,
	    set = function (v)
	         Loudmouth.db.profile.rates["Misses"] = v
				if (LMFrame:IsShown()) then Loudmouth:Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	channel = {
	    type = 'text',
	    name = L["Babble Channel"],
	    desc = L["Babble randomly in selected channel"],
	    get = function()
	        	return Loudmouth.db.profile.Channel
	    end,
	    set = function(v)
	        	Loudmouth.db.profile.Channel = v
	    end,
	    validate = { "UI", "Say", "Party", "Raid", "Guild", "Officer" },
	},
	throttle = {
	    type = 'range',
	    name = L["Wisecrack Cooldown"],
	    desc = L["Minimum time between Wisecracks."],
	    min = 5,
	    max = 300,
	    step = 5,
	    isPercent = false,
	    get = function ()
	        	return Loudmouth.db.profile.Throttle
	    end,
	    set = function (v)
	         Loudmouth.db.profile.Throttle = v
	    		Loudmouth:Update()
	    end,
	},
	afk = {
		type = 'execute',
		name = L["Show AFK Message"],
		desc = L["Show an AFK Message in Guild chat, and raid or party"],
		func = function()
		  		Loudmouth:ShowAFK()
		end,
	},
	excuse = {
		type = 'execute',
		name = L["Show Excuse"],
		desc = L["Show an excuse to Guild and default babble channel"],
		func = function()
		  		Loudmouth:ShowExcuse()
		end,
	},
	gui = {
		type = 'execute',
		name = L["Show GUI"],
		desc = L["Display GUI Window"],
		func = function()
		  		Loudmouth:ShowGUI()
		end,
	},
	--hide = {
	   --type = 'execute',
	   --name = 'Hide GUI',
	   --desc = 'Close the GUI Frame',
	   --func = "Loudmouth_Frame:Hide()",
	--},
	state = {
	   type = 'execute',
	   name = L["Show Status"],
	   desc = L["Show Command Line Status"],
		func = function()
		 		Loudmouth:ShowCmdLineStatus()
		end,
	},
  },
}

--
-- Set up ACE and Fubar menus
--

Loudmouth:RegisterChatCommand({"/loudmouth","/lm"}, options)
Loudmouth.OnMenuRequest = options

--
-- Ace / Fubar Interface handlers
--

function Loudmouth:OnClick()
  -- Left Click on FUBAR
  Loudmouth:ShowGUI()
end

--function Loudmouth:OnMenuRequest()
  --self:AddImpliedMenuOptions()
--end

function Loudmouth:OnTooltipUpdate()
  local cat = tablet:AddCategory(
    'text', L["Event Type"], 'columns', 2,
    'child_textR' , 1, 'child_textG' , 1, 'child_textB' , 0,
    'child_textR2', 1, 'child_textG2', 1, 'child_textB2', 0
  )
  if (self.db.profile.enabled["Crits"]) then
    cat:AddLine( 'text', L["Crit Hits"], 'text2', "")
  end
  if (self.db.profile.enabled["Fears"]) then
    cat:AddLine( 'text', L["Fear Effects"], 'text2', "")
  end
  if (self.db.profile.enabled["Misses"]) then
    cat:AddLine( 'text', L["Missed Attacks"], 'text2', "")
  end
  local cat = tablet:AddCategory(
    'text', L["Response Rates"], 'columns', 2,
    'child_textR' , 1, 'child_textG' , 1, 'child_textB' , 0,
    'child_textR2', 1, 'child_textG2', 1, 'child_textB2', 0
  )
  if (self.db.profile.enabled["Crits"]) then
    cat:AddLine( 'text', L["Crit Hits"], 'text2', self.db.profile.rates["Crits"])
  end
  if (self.db.profile.enabled["Fears"]) then
    cat:AddLine( 'text', L["Fear Effects"], 'text2', self.db.profile.rates["Fears"])
  end
  if (self.db.profile.enabled["Misses"]) then
    cat:AddLine( 'text', L["Missed Attacks"], 'text2', self.db.profile.rates["Misses"])
  end
  local cat = tablet:AddCategory(
    'text', L["Configuration"], 'columns', 2,
    'child_textR' , 1, 'child_textG' , 1, 'child_textB' , 0,
    'child_textR2', 0, 'child_textG2', 1, 'child_textB2', 1
  )
  cat:AddLine( 'text', L["Babble Channel"], 'text2', self.db.profile.Channel)
  cat:AddLine( 'text', L["Number of Quotes"], 'text2', self.current_total_quotes)
  cat:AddLine( 'text', L["Quote Set in Use"], 'text2', self.current_set_name)
  cat:AddLine( 'text', L["Wisecrack Cooldown"], 'text2', self.db.profile.Throttle.." "..L["seconds"])

  --local hours,minutes = GetGameTime();
  --cat:AddLine( 'text', L["Game Time"], 'text2', hours..":"..minutes)
  --cat:AddLine( 'text', L["System Uptime"], 'text2', GetTime())
  tablet:SetTitle(L["Loudmouth"])
  tablet:SetHint(L["Left click to bring up the configuration window. Right click to edit options."])
end

function Loudmouth:ShowCmdLineStatus()
  local eventTypes = ""
  local responseTypes = ""
  local channelName = ""

  for k,v in pairs(self.db.profile.enabled) do
    if (v==true)
	 then	eventTypes = eventTypes.." "..crayon:Colorize("00cf00",k)
	 else	eventTypes = eventTypes.." "..crayon:Colorize("ff0000",k)
	 end
  end
  for k,v in pairs(self.db.profile.rates) do
    if (self.db.profile.enabled[k] == true)
	 then
    responseTypes = responseTypes.." "..crayon:Colorize("00cf00",k).."="..crayon:Colorize("00cf00",v)
	 else
    responseTypes = responseTypes.." "..crayon:Colorize("ff0000",k).."="..crayon:Colorize("ff0000",v)
	 end
  end
  if (self.db.profile.Channel == "Raid")
  then channelName = crayon:Colorize("ff4f2f","Raid")
  elseif (self.db.profile.Channel == "Party")
  then channelName = crayon:Colorize("7f9fff","Party")
  elseif (self.db.profile.Channel == "Say")
  then channelName = crayon:Colorize("ffffff","Say")
  elseif (self.db.profile.Channel == "UI")
  then channelName = crayon:Colorize("ffffff","UI")
  elseif (self.db.profile.Channel == "Guild")
  then channelName = crayon:Colorize("00ff00","Guild")
  elseif (self.db.profile.Channel == "Officer")
  then channelName = crayon:Colorize("00bf00","Officer")
  end

  self:Print(L["Loudmouth"].." "..self.version.." "..L["status"]..":")
  self:Print("  "..L["Event Type"].."s:"..eventTypes)
  self:Print("  "..L["Response Rates"]..":"..responseTypes)
  self:Print("  "..L["Babble Channel"]..": "..channelName)
  self:Print("  "..L["Number of Quotes"]..": "..crayon:Colorize("9f9fff",self.current_total_quotes))
  self:Print("  "..L["Quote Set in Use"]..": "..crayon:Colorize("00bf00",self.current_set_name))
  self:Print("  "..L["Wisecrack Cooldown"]..": "..crayon:Colorize("9f9fff",self.db.profile.Throttle.." "..L["seconds"]))
end

--
-- ACE core functions
--

--function Loudmouth:OnLoad()
  --self:Debug(L["Loudmouth"].." loaded.")
--end

function Loudmouth:OnEnable()
  -- Called when the addon is enabled
  if (self.db.profile.Version ~= Loudmouth.version)
  then	self:OnUpgrade()
  			self.db.profile.Version = Loudmouth.version
  end
  LMFrame_QuoteList_Highlight_Texture:SetVertexColor(0.70, 0.70, 0.20)
  self:SetDebugging(Loudmouth.db.profile.Debug)
  self:auto_vars()
  self.LastQuoted = GetTime()
  self:RegisterMyEvents()
end

function Loudmouth:OnUpgrade()
  -- This section called if upgrading from < 0.8
  if (self.db.profile.Version < 0.8)
  then
    self:Print(L["Upgrading from below 0.8"])
    local set, event, index, value, value2, value3
    for set,value in pairs(self.db.profile.quotes) do
      self:Debug("Working in Quote Set: "..set)
		if (not self.db.profile.quoteopts[set])
		then	self.db.profile.quoteopts[set] = {}
		end
  	   for event,value2 in pairs(self.db.profile.quotes[set]) do
        self:Debug("Working in Quote Event: "..event)
		  if (not self.db.profile.quoteopts[set][event])
		  then	self.db.profile.quoteopts[set][event] = {}
		  end
	 	  for index,value3 in pairs(self.db.profile.quotes[set][event]) do
          self:Debug("Working in Quote Index: "..index)
		    if (not self.db.profile.quoteopts[set][event][index])
		    then	self.db.profile.quoteopts[set][event][index] = {}
		    end
		    if (not self.db.profile.quoteopts[set][event][index]["emote"])
		    then	self.db.profile.quoteopts[set][event][index]["emote"] = 0
		    end
		  end
	   end
    end
  end

  -- This section called if upgrading from < 0.6
  if (self.db.profile.crits)
  then	self.db.profile.enabled["Crits"] = self.db.profile.crits
  			table.remove(self.db.profile,crits)
			self:Print(L["Upgrading from below 0.6"]);
  end
  if (self.db.profile.fears)
  then	self.db.profile.enabled["Fears"] = self.db.profile.fears
  			table.remove(self.db.profile,fears)
  end
  if (self.db.profile.miss)
  then	self.db.profile.enabled["Misses"] = self.db.profile.miss
  			table.remove(self.db.profile,miss)
  end

  -- This section called if upgrading from 0.3
  if (self.db.profile.critrate ~= nil)
  then	self.db.profile.rates["Crits"]=self.db.profile.critrate
  			table.remove(self.db.profile,critrate)
			self:Print(L["Upgrading from 0.3"]);
  end
  if (self.db.profile.fearrate ~= nil)
  then	self.db.profile.rates["Fears"]=self.db.profile.fearrate
  			table.remove(self.db.profile,fearrate)
  end
  if (self.db.profile.missrate ~= nil)
  then	self.db.profile.rates["Misses"]=self.db.profile.missrate
  			table.remove(self.db.profile,missrate)
  end

  self:Print(L["Upgrading data structures from version "]..self.db.profile.Version..L[" to "]..Loudmouth.version)
end

--
-- Loudmouth UI code
--

function Loudmouth:Check_Click(box)
  if (box:GetName() == "LMEditFrame_Emote")
  then	self.db.profile.quoteopts[self.current_set_name][self.current_event_name][self.editing_item]["emote"]=box:GetChecked()
  end
  if (box:GetName() == "LMFrame_Enable")
  then
		self.db.profile.enabled[self.current_event_name] = box:GetChecked()
  end
	self:Rate_Show()
	self:Update()
end

function Loudmouth:DropDown_Initialize()
  local info, index, value, key;
  --self:Debug("Initializing Dropdown "..this:GetName())
  for index, value in pairs(Loudmouth.dropdowns.Lists) do
  	--self:Debug("Index is "..index.." Value.index is "..value.index)
	--self:Debug(this:GetName().." ".."LMFrame"..value.index.."Button")
  	if (this:GetName() == "LMFrame"..value.index.."Button" or
  	    this:GetName() == "LMFrame"..value.index)
	then
		--self:Debug("Inside IF - #value.table is "..#value.table.." maxn(table)"..table.maxn(value.table));
		local label = getglobal("LMFrame"..index.."_Label")
		--self:Debug("Label length is "..strlen(value.label))
		if (strlen(value.label) >= 2) then
			label:SetText(L[value.label])
		end
		--self:Debug("Label should be '"..L[value.label].."'")
		for key = 1, #value.table do
			info = UIDropDownMenu_CreateInfo()
			--self:Debug("checked: key="..key.." self.db.profile[index]="..self.db.profile[index])
			if (self.db.profile)
			then
				if (key == self.db.profile[index])
				then	info.checked = true
						--self:Debug("Key "..key.." = stored... "..self.db.profile[index])
				else	info.checked = nil
						--self:Debug("Key "..key.." = nil")
				end
			end
			info.text = value.table[key]
			info.func = function(x) self:DropDown_OnClick(x) end
			info.arg1 = value.index
			self:Debug("Adding 'button' "..info.text)
			UIDropDownMenu_AddButton(info)
		end
		if (string.find(this:GetName(),"_SendQuote"))
		then	local name,number
				for k = 1, 10  do
				   number,name = GetChannelName(k)
					if (name ~= nil) then
						--self:Debug("Adding Channel "..name..".")
						info = UIDropDownMenu_CreateInfo()
						info.text = name
						info.value = k
						info.func = function(x) self:DropDown_OnClick(x) end
						info.arg1 = value.index
						UIDropDownMenu_AddButton(info)
					end
				end
		end
		break;
	end
	UIDropDownMenu_SetSelectedID(this, this:GetID())
	--self:Debug("Init: this:GetName()="..this:GetName())
  end
end

function Loudmouth:DropDown_OnClick(list)
  local ddl = getglobal("LMFrame"..list)
  UIDropDownMenu_SetSelectedID(ddl, this:GetID())
  local selected = UIDropDownMenu_GetSelectedValue("LMFrame"..list)

  if (list == "_SendQuote")
  then	local channel = this:GetText()
  			if (this:GetID() ~= 1)
			then
				if (this:GetID() > #self.dropdowns.Lists["_SendQuote"].table)
				then
					SendChatMessage(LMFrame_QuoteEdit:GetText(),"CHANNEL",nil,GetChannelName(channel))
				else
  					SendChatMessage(LMFrame_QuoteEdit:GetText(),channel)
				end
    			UIDropDownMenu_ClearAll(LMFrame_SendQuote)
    			UIDropDownMenu_SetSelectedID(LMFrame_SendQuote,1)
			end
  			return
  end

  if (this:GetID() ~= self.db.profile[list])
  then	--self:Debug("Invoking ListBox_OnShow")
  	self.db.profile[list] = this:GetID()
	self:EditCancel()
	self:auto_vars()
	self:Rate_Show()
  	self:ListBox_OnShow("LMFrame"..list)
  else
    self.db.profile[list] = this:GetID()
	 self:Rate_Show()
  end
end

function Loudmouth:DropDown_OnShow()
  local listname = string.sub(this:GetName(),8)

  --self:Debug("DropDown_OnShow("..this:GetName()..")")
  --self:Debug("Confirm: listname is "..listname)
  --self:Debug(" and "..self.current_event_name)
  --UIDropDownMenu_ClearAll(this)
  if (listname == "_QuoteSet") then
		self:Debug("QuoteSet selected for show...")
  		UIDropDownMenu_SetSelectedName(this,self.current_set_name)
		self:Debug("QuoteSet shown...")
  end
  if (listname == "_QuoteEvent") then
		self:Debug("QuoteEvent selected for show...")
  		UIDropDownMenu_SetSelectedName(this,self.current_event_name)
		self:Debug("QuoteEvent shown...")
  end
  if (listname == "_SendQuote") then
		self:Debug("SendQuote selected for show...")
		self:DropDown_Initialize(this)
    	UIDropDownMenu_SetSelectedID(LMFrame_SendQuote,1)
		self:Debug("SendQuote shown...")
  end
end

function Loudmouth:EditCancel()
  LMFrame_QuoteEdit:SetText("")
  self.editing_item = 0
  LMFrame_ButtonDelete:Hide()
  LMFrame_SendQuote:Hide()
  LMEditFrame:Hide()
  LMFrame_QuoteEdit:ClearFocus()
  LMFrame_QuoteList_Highlight:Hide()
end

function Loudmouth:ListBox_OnScroll()
  local scrollbar = LMFrame_QuoteList_ScrollFrame_ScrollBar
  local buttonNo = 1
  local n = scrollbar:GetValue() - 1

  for i = 1, 15, 1 do
    getglobal("LMFrame_QuoteList_Button"..i.."_Text"):SetText("");
  end

  local temp = self.editing_item - n
  --self:Debug("Selected index is index "..temp)
  if temp <= 0 or temp >= 16 then
    LMFrame_QuoteList_Highlight:Hide();
  else
    local highlight = LMFrame_QuoteList_Highlight;
    highlight:Show();
    highlight:ClearAllPoints();
    highlight:SetPoint("TOPLEFT", "LMFrame_QuoteList_Button"..temp, "TOPLEFT", 0, 0);
  end

  LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollUpButton:Disable()
  for key = 1, #self.db.profile.quotes[self.current_set_name][self.current_event_name] do
    value = self.db.profile.quotes[self.current_set_name][self.current_event_name][key]
    if n == 0 then
      if buttonNo == 16 then
        LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Enable();
		  return
      end
      getglobal("LMFrame_QuoteList_Button"..buttonNo.."_Text"):SetText(value);
		--self:Debug("Button #"..buttonNo.." is Quote #"..key.." = '"..value.."'")
      buttonNo = buttonNo + 1;
    else
      n = n - 1;
		LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollUpButton:Enable()
    end
  end
  LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Disable()
end

function Loudmouth:ListBox_OnShow(ListBox)
  local i = 4;
  local scrollsteps = self.current_num_quotes-14
  if (scrollsteps < 1) then scrollsteps = 1 end

  LMFrame_QuoteList:EnableMouseWheel(true)
  LMFrame_QuoteList:SetScript("OnMouseWheel", function()
  	if arg1 > 0 then LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollUpButton:Click() 
	elseif arg1 < 0 then LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Click()
	end
  end)
  for i = 1, 15 do
    getglobal("LMFrame_QuoteList_Button"..i.."_Text"):SetText("")
    getglobal("LMFrame_QuoteList_Button"..i..""):Hide()
  end
  LMFrame_QuoteList_ScrollFrame:Hide()
  --self:Debug("Selected field is # "..self.db.profile["_QuoteEvent"])
  --self:Debug("Selected field name is "..event_name)
  LMFrame_QuoteList_Label:SetText(L["Quote List"].." - "..self.current_num_quotes.." "..L["quotes"]);
  --for i = 1, #self.db.profile.quotes[self.current_set_name][self.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]]
  for i = 1, #self.db.profile.quotes[self.current_set_name][self.current_event_name]
  do
  	--self:Debug("Quote # "..i.." = '"..self.db.profile.quotes[self.current_set_name][self.current_event_name][i].."'")
	if (i <= 15)
	then	getglobal("LMFrame_QuoteList_Button"..i.."_Text"):SetText(self.db.profile.quotes[self.current_set_name][self.current_event_name][i])
    		getglobal("LMFrame_QuoteList_Button"..i):Show()
	else	LMFrame_QuoteList_ScrollFrame:Show()
	end
  end
  LMFrame_QuoteList_ScrollFrame_ScrollBar:SetMinMaxValues(1,scrollsteps)
  LMFrame_QuoteList_ScrollFrame_ScrollBar:SetValue(1)
  LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollUpButton:Disable()
  if (scrollsteps > 1)
  then	LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Enable()
  else	LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Disable()
  end
end

function Loudmouth:ListButton_OnClick(button)
  local listnum=tonumber(strsub(button:GetName(),strfind(button:GetName(),"n",-3)+1))
  local quotenum=listnum+LMFrame_QuoteList_ScrollFrame_ScrollBar:GetValue()-1
  --self:Debug("Clicked button #"..listnum.." quote#"..quotenum)
  --if (quotenum <= #self.db.profile.quotes[self.current_set_name][self.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]])
  if (quotenum <= #self.db.profile.quotes[self.current_set_name][self.current_event_name])
  then
    self.editing_item=quotenum
    LMFrame_QuoteEdit:SetText(getglobal(button:GetName().."_Text"):GetText())
	 LMEditFrame_Emote:SetChecked(0)
	 LMEditFrame_Emote:SetChecked(self.db.profile.quoteopts[self.current_set_name][self.current_event_name][self.editing_item]["emote"])
    LMFrame_ButtonDelete:Show()
    LMFrame_QuoteList_Highlight:Show()
    LMFrame_QuoteList_Highlight:ClearAllPoints()
    LMFrame_QuoteList_Highlight:SetPoint("TOPLEFT",button:GetName(), "TOPLEFT", 0, 0)
	 LMFrame_QuoteEdit:SetFocus()
	 LMFrame_QuoteEdit:HighlightText(strlen(LMFrame_QuoteEdit:GetText()))
    LMFrame_SendQuote:Show()
	 LMEditFrame:Show()
  else
    self:Print("Not editing that quote, it's blank.");
  end
end

function Loudmouth:ListButton_HideTooltip(button)
  --self:Debug("Left mouseover from "..button:GetName())
  GameTooltip:Hide()
end

function Loudmouth:ListButton_ShowTooltip(button)
  --self:Debug("Got mouseover from "..button:GetName())
  local boxwidth, textwidth

  LMFrame_Temp_String_Text:SetText(getglobal(this:GetName().."_Text"):GetText())
  boxwidth = getglobal(this:GetName().."_Text"):GetStringWidth()
  --self:Debug("Box Width is "..boxwidth)
  textwidth = LMFrame_Temp_String_Text:GetWidth()
  --self:Debug("Temp Text width is "..textwidth)

  if (textwidth > boxwidth)
  then
  		GameTooltip:SetOwner(LMFrame_QuoteEdit,"ANCHOR_TOPLEFT")
  		GameTooltip:SetText(getglobal(this:GetName().."_Text"):GetText())
  end
end

function Loudmouth:Rate_Edit_Changed(Box)
  if (Box == nil) then return end
  self:Debug(self.current_event_name.." Edit Box Changed to "..Box:GetText())
  self.db.profile.rates[self.current_event_name] = tonumber(Box:GetText())
  getglobal(Box:GetParent():GetName().."_Slider"):SetValue(tonumber(Box:GetText()))
	    		self:Update()
end

function Loudmouth:Rate_Show()
  if (self.current_event_name == "Excuses" or
      self.current_event_name == "AFK")
  then	LMFrame_Rate:Hide()
  			LMFrame_Enable:Hide()
  else	
			if (self.db.profile.enabled[self.current_event_name])
			then	LMFrame_Rate:Show()
			else	LMFrame_Rate:Hide()
			end
  			LMFrame_Enable:Show()
  			LMFrame_Rate_Label:SetText(Loudmouth.current_event_name.." Rate     Slider    Value  ")
			LMFrame_Enable:SetChecked(self.db.profile.enabled[self.current_event_name])
  			LMFrame_Rate_EditBox:SetText(tonumber(self.db.profile.rates[self.current_event_name]))
  			LMFrame_Rate_SliderLow:SetText("0")
  			LMFrame_Rate_SliderHigh:SetText("100")
  			LMFrame_Rate_Slider:SetMinMaxValues(0,100)
  			LMFrame_Rate_Slider:SetValueStep(1)
  			LMFrame_Rate_Slider:SetValue(tonumber(self.db.profile.rates[self.current_event_name]))
  end
end

function Loudmouth:Rate_Slider_Changed(Slider)
  if (Slider == nil) then return end
  --self:Debug(self.current_event_name.." Slider Changed to "..Slider:GetValue())
  self.db.profile.rates[self.current_event_name] = tonumber(Slider:GetValue())
  getglobal(Slider:GetParent():GetName().."_EditBox"):SetText(tonumber(Slider:GetValue()))
  self:Update()
end

--
-- Loudmouth Logic code
--

function Loudmouth:auto_vars()
  self.current_event_id=self.db.profile["_QuoteEvent"]
  self.current_event_name=self.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]
  self.current_set_name=self.dropdowns.Lists["_QuoteSet"].table[self.db.profile["_QuoteSet"]]
  self.current_num_quotes=#self.db.profile.quotes[self.current_set_name][self.current_event_name]
   self.current_total_quotes = self:NumQuotes()
end

function Loudmouth:BeatTheOdds(rate)
  local randomvalue = math.random()

  --self:Debug("Random: "..randomvalue.." rate: "..rate)
  if (randomvalue <= (rate/100)) then
    return true
  else
    return false
  end
end

function Loudmouth:NumQuotes()
  local number = 0
  local setidx = ""
  local eventidx = ""

  setidx = next(self.db.profile.quotes, nil)
  while (setidx ~= nil)
  do
  	 eventidx = next(self.db.profile.quotes[setidx], nil)
	 while (eventidx ~= nil)
	 do
	 	--self:Debug("Quotes in "..setidx.."."..eventidx.." = "..#self.db.profile.quotes[setidx][eventidx])
		number = number + #self.db.profile.quotes[setidx][eventidx]
	 	eventidx = next(self.db.profile.quotes[setidx], eventidx)
	 end
	 setidx = next(self.db.profile.quotes, setidx)
  end
  return number
end

function Loudmouth:QuoteDelete()
  local idx = self.editing_item
  local i

  table.remove(self.db.profile.quotes[self.current_set_name][self.current_event_name],self.editing_item)
  table.remove(self.db.profile.quoteopts[self.current_set_name][self.current_event_name],self.editing_item)
  self:Update()
  LMFrame_QuoteEdit:SetText("")
  LMFrame_QuoteEdit:ClearFocus()
  LMFrame_SendQuote:Hide()
  LMEditFrame:Hide()
  LMFrame_QuoteList_Highlight:Hide()
  self:auto_vars()
  self:ListBox_OnShow("LMFrame_QuoteList")
end

function Loudmouth:QuoteEdit()
  local quote = LMFrame_QuoteEdit:GetText()
  local event = self.db.profile["_QuoteEvent"]
  local event_name = self.dropdowns.Lists["_QuoteEvent"].table[event]
  local next_index = #self.db.profile.quotes[self.current_set_name][event_name]+1
  local scrolltobottom = false;

  --self:Debug("self.editing_item is "..self.editing_item)
  if (self.editing_item == 0) then
  	self.editing_item = next_index
	scrolltobottom = true;
	--self:Debug("Choosing next_index... Next index is: "..next_index)
  end
  --self:Debug("Got a string: "..quote)
  --self:Debug("Event # is "..event.." and event_name is "..event_name)
  --self:Debug("self.editing_item is "..self.editing_item)
  if (self.editing_item <= #self.db.profile.quotes[self.current_set_name][event_name])
  then	--self:Debug("Replacing '"..self.db.profile.quotes[self.current_set_name][event_name][self.editing_item].."' with '"..quote.."'")
  elseif (strlen(quote) == 0)
  then	self:Print(L["Quote was blank. Skipping."])
  else	--self:Debug("Inserting new quote: '"..quote.."'")
  end

  if (self.editing_item > 0 and strlen(quote) > 0)
  then	--getglobal("LMFrame_QuoteList_Button"..self.editing_item.."_Text"):SetText(quote)
  	self.db.profile.quotes[self.current_set_name][event_name][self.editing_item] = quote
	if (not self.db.profile.quoteopts[self.current_set_name][self.current_event_name][self.editing_item]) then
		self.db.profile.quoteopts[self.current_set_name][self.current_event_name][self.editing_item] = {}
	end
	self.db.profile.quoteopts[self.current_set_name][self.current_event_name][self.editing_item]["emote"] = LMEditFrame_Emote:GetChecked()
	--getglobal("LMFrame_QuoteList_Button"..self.editing_item):Show()
	self.editing_item = 0
  end

  LMFrame_QuoteEdit:SetText("")
  LMFrame_QuoteList_Highlight:Hide()
  LMFrame_ButtonDelete:Hide()
  LMFrame_SendQuote:Hide()
  LMEditFrame:Hide()
  self:auto_vars()
  self:ListBox_OnShow("LMFrame_QuoteList")
  if (scrolltobottom)
  then	local _,max=LMFrame_QuoteList_ScrollFrame_ScrollBar:GetMinMaxValues()
  			LMFrame_QuoteList_ScrollFrame_ScrollBar:SetValue(max)
  			self:Quote_OnScroll()
  			LMFrame_QuoteEdit:ClearFocus()
  end
end

function Loudmouth:ShowAFK()
  local num_afks = #self.db.profile.quotes[self.current_set_name]["AFK"]
  local index = math.random(num_afks)
  	while (num_afks > 1 and self.lastQuoteID["AFK"] == index) do
		index = math.random(num_afks)
	end
  statement = self.db.profile.quotes[self.current_set_name]["AFK"][index]
  self.lastQuoteID["AFK"] = index

  SendChatMessage(statement,"AFK")
  SendChatMessage(statement,"Guild")
  if (GetNumRaidMembers() > 1) then
    SendChatMessage(statement,"Raid")
  elseif (GetNumPartyMembers() >= 1) then
    SendChatMessage(statement,"Party")
  end
end

function Loudmouth:ShowExcuse()
  local num_excuses = #self.db.profile.quotes[self.current_set_name]["Excuses"]
  local index = math.random(num_excuses)
  	while (num_excuses > 1 and self.lastQuoteID["Excuses"] == index) do
		index = math.random(num_excuses)
	end
  statement = self.db.profile.quotes[self.current_set_name]["Excuses"][index]
  self.lastQuoteID["Excuses"] = index
  self:Spew()
end

function Loudmouth:ShowGUI()
  self.db.profile.Debug = self:IsDebugging()
  LMFrame_Title:SetText(L["Loudmouth"]);
  LMFrame_QuoteList_Label:SetText(L["Quote List"]);
  LMFrame_Version:SetText(L["version"].." "..Loudmouth.version.." "..Loudmouth.date)
  LMFrame:Show();
  self:auto_vars()
  --self:ListBox_OnShow(self.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]])
end

function Loudmouth:Spew()
  if (statement == '') then return end

  local output = self.db.profile.Channel
  local orig_output = output
  
  self.LastQuoted = GetTime()
  if (GetNumRaidMembers() == 0 and output == 'Raid') then output = 'Party'; end 
  if (GetNumPartyMembers() == 0 and output == 'Party') then output = 'Say'; end

  if (orig_output ~= output) 
  then self:Debug("Downgrading from "..orig_output.." to "..output)
  end

  if (emote == 1) then output = "EMOTE" end

  if (output == "UI") then
    self:Print(statement)
  else
    SendChatMessage(statement,output)
  end

  --
  -- The following line probably doesn't belong here, I just wanted to save 
  -- this default in a function I'm sure will be called eventually
  --
  self.db.profile.Debug = self:IsDebugging()
end

function Loudmouth:Throttle()
  local now = GetTime()
  if ((now - self.LastQuoted) < self.db.profile.Throttle)
  then	self:Debug("Throttling output, said something too recently.")
  			return true
  end
  return false
end

--
-- Deprecated Code
--

--function LM_Dropdown_QuoteEvent_Init(level)
  --event_selected = UIDropDownMenu_GetSelectedValue(LMFrame_QuoteEvent)
  --level = level or 1
  --local info = UIDropDownMenu_CreateInfo();
  --Loudmouth:Debug("Initted QuoteEvent Dropdown")
  --info.text = "Crit Hits";
  --info.func = LM_Dropdown_OnClick;
  --info.value = 'crits'
  --if (selectedValue == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
  --info.text = "Missed Attacks";
  --info.value = 'miss'
  --if (selectedValue == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
  --info.text = "Mob Feared";
  --info.value = 'fear'
  --if (selectedValue == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
--end

--function LM_Dropdown_QuoteSet_Init(level)
  --quoteset_selected = UIDropDownMenu_GetSelectedValue(LMFrame_QuoteSet)
  --local info;
  --level = level or 1
  --Loudmouth:Debug("Initted QuoteSet Dropdown")
  --local info = UIDropDownMenu_CreateInfo();
  --info.text = "Default"
  --info.func = LM_Dropdown_OnClick;
  --info.value = "default"
  --if (quoteset_selected == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
  --info.text = "Mark"
  --info.value = "Mark"
  --if (quoteset_selected == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
--end

--function LM_Dropdown_OnClick()
  --local currentDropDown = UIDropDownMenu_GetCurrentDropDown()
--
  --if (strfind(this:GetName(),"LMFrame_QuoteSet")) then
    --UIDropDownMenu_SetSelectedValue(LMFrame_QuoteSet, this.value)
  --end
  --if (strfind(this:GetName(),"LMFrame_QuoteEvent")) then
    --UIDropDownMenu_SetSelectedValue(LMFrame_QuoteEvent, this.value)
  --end
  --Loudmouth:Debug("I just got a dropdown click..."..this:GetName())
--end

--function Loudmouth:GetDropValue(DropDown)
  --local dropvalue = UIDropDownMenu_GetSelectedValue("LMFrame_QuoteSet")
  --self:Debug("Current value of "..DropDown)
  --self:Debug("is "..dropvalue)
--end

--function Loudmouth:LM_Dropdown_OnShow(list)
  --if (list:GetName() == "LMFrame_QuoteSet") then
    --LMFrame_QuoteSet_Label:SetText("Quote Set")
    ----UIDROPDOWNMENU_OPEN_MENU=1;
    --UIDropDownMenu_Initialize(LMFrame_QuoteSet, LM_Dropdown_QuoteSet_Init)
    --UIDropDownMenu_SetWidth(70,LMFrame_QuoteSet)
    --UIDropDownMenu_ClearAll(LMFrame_QuoteSet)
    --UIDropDownMenu_SetSelectedID(LMFrame_QuoteSet,1)
    --self:Debug("QuoteSet Dropdown Shown")
  --elseif (list:GetName() == "LMFrame_QuoteEvent") then
    ----UIDROPDOWNMENU_OPEN_MENU=2;
    --LMFrame_QuoteEvent_Label:SetText("Event")
    --UIDropDownMenu_Initialize(LMFrame_QuoteEvent, LM_Dropdown_QuoteEvent_Init)
    --UIDropDownMenu_SetWidth(90,LMFrame_QuoteEvent)
    --UIDropDownMenu_ClearAll(LMFrame_QuoteEvent)
    --UIDropDownMenu_SetSelectedID(LMFrame_QuoteEvent,1)
    --self:Debug("QuoteEvent Dropdown Shown")
  --else
    --self:Debug("Unknown Dropdown Shown")
  --end
--end

--
-- All Event handlers are located below this comment
--

function Loudmouth:RegisterMyEvents()
  self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS")
  self:RegisterEvent("CHAT_MSG_COMBAT_SELF_MISSES")
  self:RegisterEvent("CHAT_MSG_EMOTE") -- arg1=msg arg2=player
  self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE")
  self:RegisterEvent("CHAT_MSG_SYSTEM")
  self:RegisterEvent("CHAT_MSG_TEXT_EMOTE") -- arg1=msg arg2=player
  --self:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE") -- arg1
  --self:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE") -- arg1
  --self:RegisterEvent("CHAT_MSG_SKILL") -- arg1=msg
  --self:RegisterEvent("PLAYER_LEVEL_UP") -- arg1=level arg2-9=vals
  --self:RegisterEvent("PLAYER_PVP_KILLS_CHANGED")
  --self:RegisterEvent("PLAYER_TARGET_CHANGED")
    --local name,realm = UnitName("target")
  --self:RegisterEvent("SPELLCAST_CHANNEL_START") -- arg1=ms arg2=spell
  --self:RegisterEvent("SPELLCAST_CHANNEL_STOP")
  --self:RegisterEvent("SPELLCAST_START") --arg1=spell arg2=ms
  --self:RegisterEvent("SPELLCAST_STOP")
end

function Loudmouth:CHAT_MSG_COMBAT_SELF_HITS()
  if (self:Throttle()) then return; end
  local you1, you2 = string.find(arg1, L["You"]);
  local crit, crit2 = string.find(arg1, L["crit"]);

  statement = ""
  if (self.db.profile.enabled["Crits"]) then
    if (you1 == 1 and you2 == 3 and crit) then
      local message = "Loudmouth: Critical Hit! "
      if (self:BeatTheOdds(self.db.profile.rates["Crits"])) then
        --statement = "I just got a critical hit!"
  		  local num_crits = #self.db.profile.quotes[self.current_set_name]["Crits"]
  		  local index = math.random(num_crits)
  			while (num_crits > 1 and self.lastQuoteID["Crits"] == index) do
				index = math.random(num_crits)
			end
    	  --self:Debug("Index = "..index.." num_crits = "..num_crits)
        statement = self.db.profile.quotes[self.current_set_name]["Crits"][index]
		emote = self.db.profile.quoteopts[self.current_set_name]["Crits"][index]["emote"]
			self.lastQuoteID["Crits"] = index
      else
        message = message .. " - skipped"
      end
      --UIErrorsFrame:AddMessage(message,1.0,1.0,1.0,5.0)
	   --self:Debug(message)
    else
      --UIErrorsFrame:AddMessage("Loudmouth: normal hit",1.0,1.0,1.0,5.0)
	   --self:Debug("Loudmouth: normal hit")
    end
  end
  self:Spew()
end

function Loudmouth:CHAT_MSG_COMBAT_SELF_MISSES()
  if (self:Throttle()) then return; end
  local message = "Loudmouth: miss"

  statement = ""
  if (self.db.profile.enabled["Misses"]) then
    if (self:BeatTheOdds(Loudmouth.db.profile.rates["Misses"])) then
      local num_misses = #self.db.profile.quotes[self.current_set_name]["Misses"]
      local index = math.random(num_misses)
  		while (num_misses > 1 and self.lastQuoteID["Misses"] == index) do
			index = math.random(num_misses)
		end
      --self:Debug("Index is "..index)
      statement = self.db.profile.quotes[self.current_set_name]["Misses"][index]
		emote = self.db.profile.quoteopts[self.current_set_name]["Misses"][index]["emote"]
		self.lastQuoteID["Misses"] = index
    end
  end
  --UIErrorsFrame:AddMessage(message,1.0,1.0,1.0,5.0)
  --self:Debug(message)
  self:Spew()
end

function Loudmouth:CHAT_MSG_EMOTE()
  --self:Debug("CHAT_MSG_EMOTE: "..arg1.." "..arg2);
end

function Loudmouth:CHAT_MSG_MONSTER_EMOTE()
  if (self:Throttle()) then return; end
  local message = "Loudmouth: runaway "..arg2
  local you1, you2 = string.find(arg1, L["run"]);
  local crit, crit2 = string.find(arg1, L["fear"]);

  statement = ""
  if (you1 ~= nil and crit ~= nil and self.db.profile.enabled["Fears"]) then
    if (self:BeatTheOdds(self.db.profile.rates["Fears"])) then
      local num_fears = #self.db.profile.quotes[self.current_set_name]["Fears"]
      local index = math.random(num_fears)
  		while (num_fears > 1 and self.lastQuoteID["Fears"] == index) do
			index = math.random(num_fears)
		end
      statement = self.db.profile.quotes[self.current_set_name]["Fears"][index]
		emote = self.db.profile.quoteopts[self.current_set_name]["Fears"][index]["emote"]
		self.lastQuoteID["Fears"] = index
    end
    --UIErrorsFrame:AddMessage(message,1.0,1.0,1.0,5.0)
	 --self:Debug(message)
  end
  self:Spew()
end

function Loudmouth:CHAT_MSG_SYSTEM()
  if not string.find(arg1,L["You are now AFK"]) then return end
  --self:Debug(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9)
  local message = "Loudmouth: AFK"..arg1
  --UIErrorsFrame:AddMessage(message,1.0,1.0,1.0,5.0)
  --self:Debug(message)
  self:UnregisterEvent("CHAT_MSG_SYSTEM")
  Loudmouth:ShowAFK()
  self:RegisterEvent("CHAT_MSG_SYSTEM")
end

function Loudmouth:CHAT_MSG_TEXT_EMOTE()
  self:Debug("CHAT_MSG_TEXT_EMOTE: "..arg1.." "..arg2);
end

--
-- Code Snippets and Comments I want to remember are here to the end. 
-- Probably nothing of real interest down here. :)
--

--LMF = CreateFrame("Frame",nil,UIParent)
--LMFt=LMF:CreateTexture(nil,"BACKGROUND")
--LMFt:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Background")
--LMFt:SetAllPoints(LMF)
--LMF.texture=LMFt
--shown = false

  --LMF:SetFrameStrata("BACKGROUND")
  --LMF:SetWidth(100)
  --LMF:SetHeight(100)
  --LMF:SetPoint("CENTER",0,0)
  --LMF:SetMovable(true)
  --if (shown == true) then
    --LMF:Hide()
    --shown = false
  --else
    --LMF:Show()
    --shown = true
  --end


-- UnitSex("Unit") -> 1 = unknown, 2 = male, 3 = female
--function SU_EditBox:SetText(whatCommand)
  --local editBox = SU_EditBox;
  -- send the slash command to the chat box
  --ChatEdit_SendText(editBox);
--end
-- vim: ts=3 ai nu
